// Cube.java
//
// Copyright (C) 1995 Dimension X, Inc.
//   Chris Laurel 10-28-95
//
// This code may be redistributed freely.

import java.applet.Applet;
import java.awt.*;
import dnx.geom.Matrix4;
import dnx.ice.*;


public class Cube extends Applet
{
   Graphics3D g3d;
   Model cube;
   Material material;
   Light light;
   float xangle, yangle;
   int lastX, lastY;
   Matrix4 t = new Matrix4();
   int renderMode = Model.POLYGONS;
   private boolean isRendering = false;
   Graphics3DDevice dev3d = Graphics3DDevice.getDefault3DDevice();


   public void init()
   {
      cube = dev3d.createModel(createBox(2, 2, 2));
      material = new Material();
      material.setDiffuse(0, 1, 0);
      light = new DirectionalLight();
   }

   public void start()
   {
      createGraphicsContext();
      display();
   }

   public synchronized void display()
   {
      isRendering = true;

      g3d.beginRendering();

      // Clear the contents of the frame buffer
      g3d.clear(Graphics3D.COLOR_BUFFER);

      g3d.setModelRenderMode(renderMode);

      // Set up the model transformation
      t.identity();
      // translate the cube away from the camera along the z axis
      t.translate(0, 0, -5);
      // rotate it
      t.xrotate(xangle);
      t.yrotate(yangle);

      // render
      g3d.pushLight(light);
      g3d.setMaterial(material);
      g3d.concatModelMatrix(t);
      g3d.render(cube);
      g3d.popModelMatrix();
      g3d.popLight();

      g3d.endRendering();

      repaint();
   }

   
   // Standard applet methods
   public boolean mouseDown(Event e, int x, int y)
   {
      if (e.metaDown()) {
         if (renderMode == Model.POLYGONS) renderMode = Model.LINES;
         else if (renderMode == Model.LINES) renderMode = Model.POINTS;
         else renderMode = Model.POLYGONS;
         if (!isRendering) display();
      }
      lastX = x;
      lastY = y;
      return true;
   }

   public boolean mouseDrag(Event e, int x, int y)
   {
      yangle += (float) (x - lastX) / (float) size().width * (float) Math.PI;
      xangle += (float) (y - lastY) / (float) size().height * (float) Math.PI;
      lastX = x;
      lastY = y;
      if (!isRendering)
         display();
      return true;
   }

   public void paint(Graphics g)
   {
      update(g);
   }

   public void update(Graphics g)
   {
      if (g3d != null)
	 g3d.paint(g, 0, 0);
      isRendering = false;
   }


   private void createGraphicsContext()
   {
      // Create a new 3D graphics context . . . since we're just
      //   going to be rendering a cube, we don't need a depth
      //   buffer--all necessary hidden surface removal is handled
      //   by backface culling.
      g3d = dev3d.createGraphics3D(this,
                                 Graphics3D.COLOR_BUFFER,
                                 size().width, size().height);

      // Set the clear color to be light gray (generally, the same
      //   color as the main browser window)
      g3d.setClearColor(Color.blue);

      // Enable dithering, if necessary.  This will only have an effect
      //   for a non-truecolor display.
      g3d.enableFeature(Graphics3D.DITHERING);

      g3d.setShadeMode(Graphics3D.FLAT_SHADE);
   }


   // Create a box model centered about the origin.
   ModelBuilder createBox(int width, int height, int depth)
   {
      // Create a new, empty model
      ModelBuilder m = new ModelBuilder(Model.POLYGONS,
                                        Model.BINDING_PER_FACE_GENERATE,
                                        Model.BINDING_NONE,
                                        Model.BINDING_NONE);

      // Add the vertices
      m.addVertex(-width / 2, -height / 2, depth / 2);
      m.addVertex(-width / 2, height / 2, depth / 2);
      m.addVertex(-width / 2, height / 2, -depth / 2);
      m.addVertex(-width / 2, -height / 2, -depth / 2);
      m.addVertex(width / 2, -height / 2, depth / 2);
      m.addVertex(width / 2, height / 2, depth / 2);
      m.addVertex(width / 2, height / 2, -depth / 2);
      m.addVertex(width / 2, -height / 2, -depth / 2);

      // Add the faces.
      m.quad(0, 1, 2, 3);
      m.quad(7, 6, 5, 4);
      m.quad(4, 5, 1, 0);
      m.quad(3, 2, 6, 7);
      m.quad(1, 5, 6, 2);
      m.quad(3, 7, 4, 0);

      return m;
   }

}
